<?php

namespace Teknisa\EST\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Request\Row;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Teknisa\EST\Services\ValidacoesEstoqueService;


class Est57000TransferenciaEstoqueController extends \Teknisa\EST\Controllers\TeknisaCrud {
	protected $dataSourceName = '';
	protected $dataSourceNameSave = '';
	protected $parameterBagColumns = array('');
    protected $est57000transferenciaestoqueservice;
    public $itensWithErrors = [];

    public function __construct($crudService, $parameterBag,$est57000transferenciaestoqueservice) {
        parent::__construct($crudService, $parameterBag);
        $this->est57000transferenciaestoqueservice = $est57000transferenciaestoqueservice;
        
    }

	public function find(DTO\Request\Filter $request, DTO\Response $response) {
        try {
            $rota = $request->getRoutePath();
            $name = substr($rota,1);
            
            switch ($name) {
                case 'ctrlmoviestq_transferenciaEstoque':
                    $this->parameterBagColumns =  array("P_CDFILIAL","P_CDALMOXDEST","P_CDALMOXORIG","P_CDLOCALDEST","P_CDLOCALORIG","P_DTLANCINI","P_DTLANCFIN","P_DATA");
                    break;
                case 'itlanctoest_produto_itrequesto':
                    $this->parameterBagColumns =  array("P_CDFILIAL","P_NRLANCESTQ");
                    break;
                case 'requesto_itrequesto_transferenciaesotque':
                    $this->parameterBagColumns =  array("CDFILIAL");
                    break;
                default:
                    $this->parameterBagColumns =  array("");
                    break;
            }

            $this->dataSourceName = $name;
            parent::find($request,$response);
        } catch (\Exception $e) {
            $response->addDataSet(new DataSet("itensWithErrors", $this->itensWithErrors));
            $response->setError(new Error($e->getMessage(), 500));
        }
    }

    public function buscaDataProcessamento(DTO\Request $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            $data = $this->est57000transferenciaestoqueservice->buscaDataProcessamento($row);
            $dataSet = new DataSet('dataProcessamento', $data);
            $response->addDataSet($dataSet);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }

    public function editTransferencia(DTO\Request $request, DTO\Response $response) {
        try {
            $this->dataSourceName ='ctrlmoviestq_transferenciaEstoque';
            $this->dataSourceNameToSave = 'ctrlmoviestq';
            parent::save($request,$response);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }

    public function salvarNovaTransferencia(DTO\Request $request, DTO\Response $response) {
        try {
            $transferencia = $request->getRow()['transferencia'];
            $produtos = $request->getRow()['produtos'];
            $dataSet = $this->est57000transferenciaestoqueservice->salvarNovaTransferencia($transferencia,$produtos);
            
            $rows = $dataSet->getRows();
            
            $dataTransfer = new Dataset("transferencia",$rows);
            $response->addDataSet($dataTransfer);
            
        } catch (\Exception $e) {
            
            $irregularProducts = $this->est57000transferenciaestoqueservice->getIrregularProductsList();
            if(!empty($irregularProducts)) {
                $products = array();
                foreach($irregularProducts as $irregularProduct) {
                    $row = ["cdproduto"=> $irregularProduct->getCdproduto(),
                            "cdalmoxarife"=>$irregularProduct->getCdalmoxarife(),
                            "cdlocalestoq"=>$irregularProduct->getCdlocalestoq(),
                            "nrloteestq"=>$irregularProduct->getNrloteestq(),
                            "nrsublote"=>$irregularProduct->getNrsublote()];
                            
                    $products[] = $row;
                }
                $dataSet = new DataSet("irregularProducts", $products);
                $response->addDataset($dataSet);
            }
            $response->setError(new Error($e->getMessage(), 500));
        }
    }

    public function salvarNovoProduto(DTO\Request $request, DTO\Response $response) {
        try {
            $transferencia = $request->getRow()['transferencia'];
            $produto       = $request->getRow()['produtos'];

            $this->est57000transferenciaestoqueservice->salvarNovoProduto($transferencia,$produto);

        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }

    public function salvarNovaTransferenciaComRequisicao(DTO\Request$request, DTO\Response $response) {
        try {
            $transferencia = $request->getRow()['transferencia'];
            $itens = $request->getRow()["itens"];
            $dataSet = $this->est57000transferenciaestoqueservice->salvarNovaTransferenciaComRequisicao($transferencia, $itens);
            $response->addDataSet($dataSet);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }

    public function editarProdutoTransferencia(DTO\Request $request, DTO\Response $response) {
        try {
            $rows = $request->getRow();
            $dataSet = $this->est57000transferenciaestoqueservice->editProdutoTransferencia($rows);
            $response->addDataSet($dataSet);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }

    public function deletarProdutoTransferencia(DTO\Request $request, DTO\Response $response) {
        try {
            $rows = $request->getRow();
            $this->est57000transferenciaestoqueservice->deleteProduto($rows);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }

    public function deletarTransferencia(DTO\Request $request, DTO\Response $response) {
        try {
            $rows = $request->getRow();
            $this->est57000transferenciaestoqueservice->deleteTransferencia($rows);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
    public function getAlmoxRequisicao(DTO\Request $request, DTO\Response $response) {
        try {
          $dataSet = $this->est57000transferenciaestoqueservice->getAlmoxRequisicao($request->getRow()['NRREQUESTO'], $request->getRow()['CDFILIAL']);
          $response->addDataSet($dataSet);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
        
    }
    
    public function save(DTO\Request $request, DTO\Response $response) {
        $this->dataSourceName       = 'ctrlmoviestq_transferenciaEstoque';
        $this->dataSourceNameToSave = 'ctrlmoviestq_transferenciaEstoque';
        parent::save($request, $response);
    }
    
    public function importarProdutosTxt(Row $request, Response $response){
        $fileType = 'txt';
        $row = $request->getRow();
        if (isset($row['UPLOADFILE'])){
            $fileType = 'xls';
        }
        
        
        if($fileType === 'txt'){
            $isTxt = strpos($row['ARQUIVOTXT'][0]['type'],'text/plain') !== false;
            $fileType = 'txt';
            $arquivoB64 = $row["ARQUIVOTXT"][0]["b64File"];
            if(strpos($arquivoB64, ",") !== false){
                $arquivoB64 = explode(",", $arquivoB64)[1];
            }
            
            $arquivo = utf8_encode(base64_decode($arquivoB64));
            $produtos = explode("\n", $arquivo);
            $produtos = array_filter($produtos, function($v){
                return $v != "" && $v != null; 
            });
            
            if($arquivoB64 == null || $arquivoB64 == "" || $arquivoB64 == "data:"){
                throw new \Exception("Operação bloqueada! Arquivo de Origem deve ser informado.",400);
            }
        }else{
            $isTxt = strpos($row['UPLOADFILE'][0]['type'],'text/plain') !== false;
            $row['CAMPOS'] = $row['PARAMS']['CAMPOS'];
            $table = $this->est57000transferenciaestoqueservice->readXls($row);
            $produtos = $this->est57000transferenciaestoqueservice->formatXls($table);
        }
        
        
        if ($fileType === 'txt'){
            try{
                $params = $row["PARAMS"];
                $itensTransferencia = [];
                $ErrorRowNumber = 1;
                foreach($produtos as &$produto){
                    $produto = $this->est57000transferenciaestoqueservice->buildObjectTheCode($produto, $params);
                    $ErrorRowNumber++;
                    $produto = $this->est57000transferenciaestoqueservice->completaProduto($produto, $row['CDFILIAL'], $row["CDFILIMOVI"], $row["DTMOVIESTQ"],  $ErrorRowNumber, $fileType);
                    $itensTransferencia[] = $produto;
                }
                
                $this->dataSourceName = 'lanctoestoq_tranferencia';
                $response->addDataSet(new DataSet("itensTransferencia", $itensTransferencia));
            } catch(\Exception $e) {
                $response->setError(new Error($e->getMessage(), 500));
            }
        }
        
        else{
            
            $itTransferencia = [];
            try{
                $params = $row["PARAMS"];
                $ErrorRowNumber = 1;
                foreach($produtos as &$produto){
                    $produto = $this->est57000transferenciaestoqueservice->buildObjectTheCodeXls($produto, $params, $this->itensWithErrors, $ErrorRowNumber);
                    $ErrorRowNumber++;
                    $produto = $this->est57000transferenciaestoqueservice->completaProduto($produto, $row['CDFILIAL'], $row["CDFILIMOVI"], $row["DTMOVIESTQ"], $ErrorRowNumber, $fileType);
                    $itensTransferencia[] = $produto;
                }
                
                
                
                $this->dataSourceName = 'lanctoestoq_tranferencia';
                $response->addDataSet(new DataSet("itensTransferencia", $itensTransferencia));
            } catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
            }
        }
    }

}